@@ -14,7 +14,7 @@
 #' \code{propensity.func = function(x, trt) 0.5}.
 #' @param loss choice of both the M function from Chen, et al (2017) and potentially the penalty used for variable selection.
 #' All \code{loss} options starting with \code{sq_loss} use M(y, v) = (v - y) ^ 2, all options starting with \code{logistic_loss} use
-#' the logistic loss: M(y, v) = y * log(1 + exp{-v}), and all options starting with \code{cox_loss} use the negative partial likelihood loss for the Cox PH model.
+#' the logistic loss: M(y, v) = y * log(1 + exp\{-v\}), and all options starting with \code{cox_loss} use the negative partial likelihood loss for the Cox PH model.
 #' All options ending with \code{lasso} have a lasso penalty added to the loss for variable selection. \code{sq_loss_lasso_gam}
 #' and \code{logistic_loss_lasso_gam} first use the lasso to select variables and then fit a generalized additive model
 #' with nonparametric additive terms for each selected variable. \code{sq_loss_gam} involves a squared error loss with a generalized additive model and no variable selection.
@@ -26,16 +26,24 @@
 #'     \item{\code{"logistic_loss_lasso"}}{ - M(y, v) = -[yv - log(1 + exp\{-v\})] with with linear model and lasso penalty}
 #'     \item{\code{"cox_loss_lasso"}}{ - M corresponds to the negative partial likelihood of the cox model with linear model and additionally a lasso penalty}
 #'     \item{\code{"sq_loss_lasso_gam"}}{ - M(y, v) = (v - y) ^ 2 with variables selected by lasso penalty and generalized additive model fit on the selected variables}
-#'     \item{\code{"logistic_loss_lasso_gam"}}{ - M(y, v) = y * log(1 + exp{-v}) with variables selected by lasso penalty and generalized additive model fit on the selected variables}
+#'     \item{\code{"logistic_loss_lasso_gam"}}{ - M(y, v) = y * log(1 + exp\{-v\}) with variables selected by lasso penalty and generalized additive model fit on the selected variables}
 #'     \item{\code{"sq_loss_gam"}}{ - M(y, v) = (v - y) ^ 2 with generalized additive model fit on all variables}
-#'     \item{\code{"logistic_loss_gam"}}{ - M(y, v) = y * log(1 + exp{-v}) with generalized additive model fit on all variables}
+#'     \item{\code{"logistic_loss_gam"}}{ - M(y, v) = y * log(1 + exp\{-v\}) with generalized additive model fit on all variables}
 #'     \item{\code{"sq_loss_gbm"}}{ - M(y, v) = (v - y) ^ 2 with gradient-boosted decision trees model}
 #'     \item{\code{"abs_loss_gbm"}}{ - M(y, v) = |v - y| with gradient-boosted decision trees model}
-#'     \item{\code{"huberized_loss_gbm"}}{ - M corresponds to Huberized hinge loss (for binary outcomes) with gradient-boosted decision trees model}
 #'     \item{\code{"logistic_loss_gbm"}}{ - M(y, v) = -[yv - log(1 + exp\{-v\})] with gradient-boosted decision trees model}
 #'     \item{\code{"cox_loss_gbm"}}{ - M corresponds to the negative partial likelihood of the cox model with gradient-boosted decision trees model}
 #' }
 #' @param method subgroup ID model type. Either the weighting or A-learning method of Chen et al, (2017)
+#' @param match.id a (character, factor, or integer) vector with length equal to the number of observations in \code{x} indicating using integers or
+#' levels of a factor vector which patients are
+#' in which matched groups. Defaults to \code{NULL} and assumes the samples are not from a matched cohort. Matched
+#' case-control groups can be created using any method (propensity score matching, optimal matching, etc). If each case
+#' is matched with a control or multiple controls, this would indicate which case-control pairs or groups go together.
+#' If \code{match.id} is supplied, then it is unecessary to specify a function via the \code{propensity.func} argument.
+#' A quick usage example: if the first patient is a case and the second and third are controls matched to it, and the
+#' fouth patient is a case and the fifth through seventh patients are matched with it, then the user should specify
+#' \code{match.id = c(1,1,1,2,2,2,2)} or \code{match.id = c(rep("Grp1", 3),rep("Grp2", 4)) }
 #' @param augment.func function which inputs the response \code{y}, the covariates \code{x}, and \code{trt} and outputs
 #' predicted values for the response using a model constructed with \code{x}. \code{augment.func()} can also be simply
 #' a function of \code{x} and \code{y}. This function is used for efficiency augmentation.
@@ -48,18 +56,21 @@
 #' will be recommended to be in the treatment group
 #' @param larger.outcome.better boolean value of whether a larger outcome is better/preferable. Set to \code{TRUE}
 #' if a larger outcome is better/preferable and set to \code{FALSE} if a smaller outcome is better/preferable. Defaults to \code{TRUE}.
+#' @param reference.trt which treatment should be treated as the reference treatment. Defaults to the first level of \code{trt}
+#' if \code{trt} is a factor or the first alphabetical or numerically first treatment level.
 #' @param retcall boolean value. if \code{TRUE} then the passed arguments will be saved. Do not set to \code{FALSE}
-#' if the \code{validate.subgroupu()} function will later be used for your fitted subgroup model. Only set to \code{FALSE}
-#' if memory is limited as setting to code{TRUE} saves the design matrix to the fitted object
+#' if the \code{validate.subgroup()} function will later be used for your fitted subgroup model. Only set to \code{FALSE}
+#' if memory is limited as setting to \code{TRUE} saves the design matrix to the fitted object
 #' @param ... options to be passed to underlying fitting function. For all \code{loss} options with \code{lasso},
 #' this will be passed to \code{cv.glmnet} and for all \code{loss} options with \code{mcp} this will be passed
 #' to \code{cv.ncvreg}. Note that for all \code{loss} options that use \code{gam()} from the \code{mgcv} package,
 #' the user cannot supply the \code{gam} argument \code{method} because it is also an argument of \code{fit.subgroup}, so
-#' instead, to change the \code{gam method} argument, instead supply \code{method.gam}, ie \code{method.gam = "REML"}.
+#' instead, to change the \code{gam method} argument, supply \code{method.gam}, ie \code{method.gam = "REML"}.
 #' @seealso \code{\link[personalized]{validate.subgroup}} for function which creates validation results for subgroup
 #' identification models, \code{\link[personalized]{predict.subgroup_fitted}} for a prediction function for fitted models
-#' from \code{fit.subgroup}, and \code{\link[personalized]{plot.subgroup_fitted}} for a function which plots
-#' results from fitted models
+#' from \code{fit.subgroup}, \code{\link[personalized]{plot.subgroup_fitted}} for a function which plots
+#' results from fitted models, and \code{\link[personalized]{print.subgroup_fitted}}
+#' for arguments for printing options for \code{fit.subgroup()}.
 #' from \code{fit.subgroup}.
 #' @references Chen, S., Tian, L., Cai, T. and Yu, M. (2017), A general statistical framework for subgroup identification
 #' and comparative treatment scoring. Biometrics. doi:10.1111/biom.12676 \url{http://onlinelibrary.wiley.com/doi/10.1111/biom.12676/abstract}
@@ -150,7 +161,7 @@
 fit.subgroup <- function(x,
                          y,
                          trt,
-                         propensity.func,
+                         propensity.func = NULL,
                          loss       = c("sq_loss_lasso",
                                         "logistic_loss_lasso",
                                         "cox_loss_lasso",
@@ -160,13 +171,14 @@ fit.subgroup <- function(x,
                                         "logistic_loss_gam",
                                         "sq_loss_gbm",
                                         "abs_loss_gbm",
-                                        "huberized_loss_gbm",
                                         "logistic_loss_gbm",
                                         "cox_loss_gbm"),
                          method     = c("weighting", "a_learning"),
+                         match.id = NULL,
                          augment.func = NULL,
                          cutpoint   = 0,
                          larger.outcome.better = TRUE,
+                         reference.trt = NULL,
                          retcall    = TRUE,
                          ...)
 {
@@ -174,50 +186,52 @@ fit.subgroup <- function(x,
     loss   <- match.arg(loss)
     method <- match.arg(method)
 
-    # set default family and change later
-    # if the loss argument is set to something
-    # not for continuous outcomes
-    family <- "gaussian"
+
+    # make sure outcome is consistent with
+    # other options selected if there is any
+    # indication of a survival outcome or model
+    if ( xor(class(y) == "Surv", grepl("cox_loss", loss)) )
+    {
+        ifelse(
+            grepl("cox_loss", loss),
+            stop("Must provide 'Surv' object if loss/family corresponds to a Cox model. See\n
+                 '?Surv' for more information about 'Surv' objects."),
+            stop("Loss and family must correspond to a Cox model for time-to-event outcomes.")
+            )
+    }
+
+    if (grepl("cox_loss", loss))
+    {
+        family <- "cox"
+    } else if (grepl("logistic_loss", loss) | grepl("huberized_loss", loss))
+    {
+        family <- "binomial"
+    } else
+    {
+        family <- "gaussian"
+    }
+
 
     dims   <- dim(x)
     if (is.null(dims)) stop("x must be a matrix object.")
 
     y      <- drop(y)
     vnames <- colnames(x)
+
+    if (!is.null(match.id))
+    {
+        if (length(match.id) != dims[1L]) stop("match.id must be same length as number of observations.")
+    }
+
+    # set variable names if they are not set
     if (is.null(vnames)) vnames <- paste0("V", 1:dims[2])
 
+    ## will be a flag for later use if
+    ## I decide to make outcome-weighted learning
+    ## (ie flipping loss) an option
     outcome.weighted <- FALSE
 
-    # make sure outcome is consistent with
-    # other options selected
-    if (class(y) == "Surv")
-    {
-        if (length(grep("cox_loss", loss)) == 0)
-        {
-            stop("Loss and family must correspond to a Cox model for time-to-event outcomes.")
-        } else
-        {
-            family <- "cox"
-        }
-    }
 
-    if (length(grep("cox_loss", loss)) > 0)
-    {
-        if (class(y) != "Surv")
-        {
-            stop("Must provide 'Surv' object if loss/family corresponds to a Cox model. See
-                 '?Surv' for more information about 'Surv' objects.")
-        } else
-        {
-            family <- "cox"
-        }
-    }
-
-    if (length(grep("logistic_loss", loss)) > 0 |
-        length(grep("huberized_loss", loss)) > 0)
-    {
-        family <- "binomial"
-    }
 
 
 
@@ -244,6 +258,109 @@ fit.subgroup <- function(x,
             stop("augment.func() should only have either two arguments: 'x' and 'y', or three arguments:
                  'trt', 'x', and 'y'")
         }
+        }
+
+    if (is.factor(trt))
+    {
+        # drop any unused levels of trt
+        trt         <- droplevels(trt)
+        unique.trts <- levels(trt)
+        n.trts      <- length(unique.trts)
+    } else
+    {
+        unique.trts <- sort(unique(trt))
+        n.trts      <- length(unique.trts)
+    }
+
+    if (n.trts < 2)           stop("trt must have at least 2 distinct levels")
+    if (n.trts > dims[1] / 3) stop("trt must have no more than n.obs / 3 distinct levels")
+
+    if (!is.null(reference.trt))
+    {
+        if (!(reference.trt %in% unique.trts))
+        {
+            stop("reference.trt must be one of the treatment levels")
+        }
+
+        reference.idx   <- which(unique.trts == reference.trt)
+        comparison.idx  <- (1:n.trts)[-reference.idx]
+        comparison.trts <- unique.trts[-reference.idx]
+
+    } else
+    {
+        reference.idx   <- 1L
+        comparison.idx  <- (1:n.trts)[-reference.idx]
+        comparison.trts <- unique.trts[-reference.idx]
+        reference.trt   <- unique.trts[reference.idx]
+    }
+
+    if (n.trts > 2 & (grepl("_gbm", loss) | grepl("_gam", loss)) )
+    {
+        stop("gbm and gam based losses not supported for multiple treatments (number of total treatments > 2)")
+    }
+
+    # Check match.id validity and convert it to a factor, if supplied
+    if (!is.null(match.id)) {
+        match.id <- tryCatch(expr=as.factor(match.id), error = function(e) {stop("match.id must be a factor or capable of being coerced to a factor.")})
+        if (length(levels(match.id)) < 2) {stop("match.id must have at least 2 levels")}
+    }
+
+    # defaults to constant propensity score within trt levels
+    # the user will almost certainly want to change this
+    if (is.null(propensity.func))
+    {
+        if (is.null(match.id))
+        { # No propensity score supplied and no match.id supplied
+            if (n.trts == 2)
+            {
+                mean.trt <- mean(trt == unique.trts[2L])
+                propensity.func <- function(trt, x) rep(mean.trt, length(trt))
+            } else
+            {
+                mean.trt <- numeric(n.trts)
+                for (t in 1:n.trts)
+                {
+                    mean.trt[t] <- mean(trt == unique.trts[t])
+                }
+                propensity.func <- function(trt, x)
+                {
+                    pi.x <- numeric(length(trt))
+                    for (t in 1:n.trts)
+                    {
+                        which.t       <- trt == unique.trts[t]
+                        pi.x[which.t] <- mean(which.t)
+                    }
+                    pi.x
+                }
+            }
+        } else
+        { # No propensity score supplied but match.id supplied
+            if (n.trts == 2)
+            {
+                # default to pct in treatment group
+                mean.trt <- mean(trt == unique.trts[2L])
+                pf <- function(trt, x, match.id) rep(mean.trt, NROW(trt))
+                propensity.func <- pf
+            } else
+            {
+                mean.trt <- numeric(n.trts)
+                for (t in 1:n.trts)
+                {
+                    mean.trt[t] <- mean(trt == unique.trts[t])
+                }
+                pf <- function(trt, x, match.id)
+                {
+                    pi.x <- numeric(length(trt))
+                    for (t in 1:n.trts)
+                    {
+                        which.t       <- trt == unique.trts[t]
+                        pi.x[which.t] <- mean(which.t)
+                    }
+                    pi.x
+                }
+                propensity.func <- pf
+            }
+        }
     }
 
 
@@ -263,48 +380,55 @@ fit.subgroup <- function(x,
         y.adj <- y
     }
 
+    # stop if augmentation function provided
+    # for non-gaussian outcomes.
+    # has not been developed yet
     if (!is.null(augment.func) & family != "gaussian")
     {
         warning("Efficiency augmentation not available for non-continuous outcomes yet. No augmentation applied.")
     }
 
     larger.outcome.better <- as.logical(larger.outcome.better[1])
-    retcall <- as.logical(retcall[1])
+    retcall               <- as.logical(retcall[1])
 
     # save the passed arguments for later use in validate.subgroupu()
     # and plot.subgroup_fitted() functions
     if (retcall)
     {
-        this.call <- mget(names(formals()), sys.frame(sys.nframe()))
+        this.call     <- mget(names(formals()), sys.frame(sys.nframe()))
 
         this.call$... <- NULL
-        this.call <- c(this.call, list(...))
+        this.call     <- c(this.call, list(...))
     } else
     {
-        this.call <- NULL
+        this.call     <- NULL
     }
 
-    trt         <- as.integer(trt)
-    unique.trts <- sort(unique(trt))
-
-    if (length(unique.trts) != 2)    stop("trt must have 2 distinct levels")
-    if (any(unique.trts != c(0, 1))) stop("trt should be coded as 0 and 1")
-
     # check to make sure arguments of propensity.func are correct
     propfunc.names <- sort(names(formals(propensity.func)))
-    if (length(propfunc.names) == 2)
+    if (length(propfunc.names) == 3)
+    {
+        if (any(propfunc.names != c("match.id", "trt", "x")))
+        {
+            stop("arguments of propensity.func() should be 'trt','x', and (optionally) 'match.id'")
+        }
+    } else if (length(propfunc.names) == 2)
     {
         if (any(propfunc.names != c("trt", "x")))
         {
-            stop("arguments of propensity.func() should be 'x' and 'trt'")
+            stop("arguments of propensity.func() should be 'trt','x', and (optionally) 'match.id'")
         }
     } else
     {
-        stop("propensity.func() should only have two arguments: 'trt' and 'x'")
+        stop("propensity.func() should only have two or three arguments: 'trt' and 'x', or: 'trt', 'x', and 'match.id'")
     }
 
     # compute propensity scores
-    pi.x <- propensity.func(x = x, trt = trt)
+    if (is.null(match.id)) {
+        pi.x <- drop(propensity.func(x = x, trt = trt))
+    } else {
+        pi.x <- drop(propensity.func(x = x, trt = trt, match.id = match.id))
+    }
 
     # make sure the resulting propensity scores are in the
     # acceptable range (ie 0-1)
@@ -312,42 +436,108 @@ fit.subgroup <- function(x,
 
     if (rng.pi[1] <= 0 | rng.pi[2] >= 1) stop("propensity.func() should return values between 0 and 1")
 
-    trt2 <- 2 * trt - 1
+    ## if returned propensity score
+    ## is a matrix, then pick out the
+    ## right column for each row so we
+    ## always get Pr(T = T_i | X = x)
 
-    # construct modified design matrices
-    # and weights
-    # depending on what method is used
-    if (method == "weighting")
+    dim.pi.x <- dim(pi.x)
+    if (!is.null(dim.pi.x))
     {
-        x.tilde <- trt2 * cbind(1, x)
-        wts     <- 1 / (pi.x * (trt == 1) + (1 - pi.x) * (trt == 0))
-    } else
-    {   # A-learning method
-        x.tilde <- (trt - pi.x) * cbind(1, x)
-        wts     <- rep(1, nrow(x))
+        if (length(dim.pi.x) == 1)
+        {
+            pi.x <- as.vector(pi.x)
+        } else if (length(dim.pi.x) == 2)
+        {
+            if (ncol(pi.x) != n.trts)
+            {
+                stop("Number of columns in the matrix returned by propensity.func() is not the same
+                     as the number of levels of 'trt'.")
+            }
+            if (is.factor(trt))
+            {
+                values <- levels(trt)[trt]
+            } else
+            {
+                values <- trt
+            }
+
+            levels.pi.mat <- colnames(pi.x)
+            if (is.null(levels.pi.mat))
+            {
+                levels.pi.mat <- unique.trts
+            }
+
+            # return the probability corresponding to the
+            # treatment that was observed
+            pi.x <- pi.x[cbind(1:nrow(pi.x), match(values, levels.pi.mat))]
+            } else
+            {
+                stop("propensity.func() returns a multidimensional array; it can only return a vector or matrix.")
+            }
     }
 
-    colnames(x.tilde) <- c("Trt", vnames)
+    # construct design matrix to be passed to fitting function
+    x.tilde <- create.design.matrix(x             = x,
+                                    pi.x          = pi.x,
+                                    trt           = trt,
+                                    method        = method,
+                                    reference.trt = reference.trt)
+
+    # construct observation weight vector
+    wts     <- create.weights(pi.x   = pi.x,
+                              trt    = trt,
+                              method = method)
+
+    if (n.trts > 2)
+    {
+        all.cnames <- numeric(ncol(x.tilde))
+        len.names  <- length(vnames) + 1
+        for (tr in 1:(n.trts - 1))
+        {
+            idx.cur <- ((len.names * (tr - 1)) + 1):(len.names * tr)
+            all.cnames[idx.cur] <- c( comparison.trts[tr],
+                                      paste(vnames, 1:(n.trts - 1), sep = ".") )
+        }
+    } else
+    {
+        all.cnames <- c( comparison.trts,
+                         vnames )
+    }
+
+    colnames(x.tilde) <- all.cnames
 
     # identify correct fitting function and call it
     fit_fun      <- paste0("fit_", loss)
-    fitted.model <- do.call(fit_fun, list(x = x.tilde, y = y.adj, wts = wts, family = family, ...))
+    fitted.model <- do.call(fit_fun, list(x = x.tilde, trt = trt, n.trts = n.trts,
+                                          y = y.adj, wts = wts, family = family,
+                                          match.id = match.id, ...))
+
 
     # save extra results
-    fitted.model$call   <- this.call
-    fitted.model$family <- family
-    fitted.model$loss   <- loss
-    fitted.model$method <- method
+    fitted.model$call                  <- this.call
+    fitted.model$family                <- family
+    fitted.model$loss                  <- loss
+    fitted.model$method                <- method
     fitted.model$larger.outcome.better <- larger.outcome.better
-    fitted.model$var.names <- vnames
-    fitted.model$benefit.scores <- fitted.model$predict(x)
+    fitted.model$var.names             <- vnames
+    fitted.model$n.trts                <- n.trts
+    fitted.model$comparison.trts       <- comparison.trts
+    fitted.model$reference.trt         <- reference.trt
+
+    fitted.model$benefit.scores        <- fitted.model$predict(x)
+
+    fitted.model$recommended.trts      <- predict.subgroup_fitted(fitted.model, newx = x,
+                                                                  type = "trt.group",
+                                                                  cutpoint = cutpoint)
 
     # calculate sizes of subgroups and the
     # subgroup treatment effects based on the
     # benefit scores and specified benefit score cutpoint
     fitted.model$subgroup.trt.effects <- subgroup.effects(fitted.model$benefit.scores,
                                                           y, trt, cutpoint,
-                                                          larger.outcome.better)
+                                                          larger.outcome.better,
+                                                          reference.trt = reference.trt)
 
     class(fitted.model) <- "subgroup_fitted"
 
